unit softrender;

interface

uses Windows,Graphics,JPEG,dialogs,sysutils;
                    
type TTexture=record
              FrameBuffer:Pointer;
              width,height:integer;
             end;
var Textures:array of TTexture;

function LoadTexture(fname:string):integer;
procedure DrawLine(id1,id2:integer);
procedure proj3Dto2D(x,y,z, zEye:single; var x0,y0:integer);

var ww,hh:integer;
var xofs,yofs:integer;
var BitmapInfo:TBitmapInfo;
var FrameBuffer,DepthBuffer:Pointer;

var ptr_screen_pixel:^Cardinal;
var ptr_screen_depth:^Cardinal;
var ptr_texture_pixel:^Cardinal;

type TRasterVect=packed record
                  x,y:integer;
                  z:integer;
                  u,v:integer;
                 end;
var _p:array[0..2] of TRasterVect;

type TScanPoint=record
                 _x1,_x2: integer;
                 _u1,_v1,_z1: integer;
                 _u2,_v2,_z2: integer;

                end;

var Scanlines:array[0..1023] of TScanPoint;

var z_lookup:array[1..$FFFFF] of integer;

const wide1=16;
const wide2=24; //$FFFFF
//11111111111111111111


implementation

type
  TRGBArray = Array[0..32767] of TRGBQuad; // rgba
  pRGBArray = ^TRGBArray;

///////////////////////////////////////////////////////////
// Load texture 'fname.*', returns index in array Textures[]
///////////////////////////////////////////////////////////
function LoadTexture(fname:string):integer;
var i,j,k:integer;
var Picture: TPicture;
var sl: pRGBArray;  {Scanline}
var bmp:TBitmap;
begin

 Picture := TPicture.Create;
 Picture.LoadFromFile(fname);
 bmp:=TBitmap.Create;
 bmp.Width:=Picture.Width;
 bmp.Height:=Picture.Height;
 bmp.PixelFormat:=pf32bit;
 bmp.Canvas.Draw(0,0,Picture.Graphic);
 Picture.Free;

 k:=length(Textures);
 setlength(Textures,k+1);
 reallocmem(Textures[k].FrameBuffer,bmp.Width*bmp.Height*4);
 Textures[k].width:=bmp.Width;
 Textures[k].height:=bmp.Height;

 ptr_texture_pixel:=pointer(Textures[k].FrameBuffer);

 for j := 0 to bmp.Height - 1 do
  begin
   sl := bmp.ScanLine[j];
   for i := 0 to bmp.Width - 1 do
    begin
     ptr_texture_pixel^:=Cardinal(sl[i]);
     inc(ptr_texture_pixel,1);
    end;
  end;

 result:=k;

end;




///////////////////////////////////////////////////////////
// Draw side of polygon from point with id1 to id2 (array _p)
///////////////////////////////////////////////////////////
procedure DrawLine(id1,id2:integer);
var sn:integer;
var dx,dy,du,dv,dz:integer;
var x,u,v,z:integer;
var y:integer;


begin

 if _p[id1].y=_p[id2].y then exit;
 if _p[id2].y<_p[id1].y then
  begin
   sn:=id1;
   id1:=id2;
   id2:=sn;
  end;

  x:=_p[id1].x;
  u:=_p[id1].u;
  v:=_p[id1].v;
  z:=_p[id1].z;

///        showmessage(inttostr(u)+' '+inttostr(v));


  dy:=(_p[id2].y-_p[id1].y);
  dx:=(_p[id2].x-_p[id1].x) div dy;
  du:=(_p[id2].u-_p[id1].u) div dy;
  dv:=(_p[id2].v-_p[id1].v) div dy;
  dz:=(_p[id2].z-_p[id1].z) div dy;



  for y := _p[id1].y to _p[id2].y do
   begin


     if (y>0) and (y<hh) then
      begin


      if (x shr wide1 < Scanlines[y]._x1) then
       begin
        Scanlines[y]._x1:=abs(x) shr wide1;
        Scanlines[y]._u1:=u;
        Scanlines[y]._v1:=v;
        Scanlines[y]._z1:=z;
       end;
      if (x shr wide1 > Scanlines[y]._x2) then
       begin
        Scanlines[y]._x2:=abs(x) shr wide1;
        Scanlines[y]._u2:=u;
        Scanlines[y]._v2:=v;
        Scanlines[y]._z2:=z;
       end;

//        showmessage(inttostr(u)+' '+inttostr(v));

      end;

    x:=x+dx;
    u:=u+du;
    v:=v+dv;
    z:=z+dz;
   end;

end;


procedure proj3Dto2D(x,y,z, zEye:single; var x0,y0:integer);
begin
 X0:= Xofs  +  Round(X*(Zeye/(Zeye-Z)));
 Y0:= Yofs  +  Round(Y*(Zeye/(Zeye-Z)));
end;

var _i:integer;
begin

 for _i := 1 to length(z_lookup) do
    z_lookup[_i]:=round((1 shl wide2{ffffff}) /_i);



end.
